
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "gedit-enum-types.h"
#include "gedit-app-activatable.h"
#include "gedit-app.h"
#include "gedit-commands.h"
#include "gedit-debug.h"
#include "gedit-document.h"
#include "gedit-encodings-combo-box.h"
#include "gedit-menu-extension.h"
#include "gedit-message-bus.h"
#include "gedit-message.h"
#include "gedit-tab.h"
#include "gedit-utils.h"
#include "gedit-view-activatable.h"
#include "gedit-view.h"
#include "gedit-window-activatable.h"
#include "gedit-window.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "gedit-debug.h" */

GType
gedit_debug_section_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GEDIT_NO_DEBUG), "GEDIT_NO_DEBUG", "no-debug" },
        { C_FLAGS (GEDIT_DEBUG_WINDOW), "GEDIT_DEBUG_WINDOW", "debug-window" },
        { C_FLAGS (GEDIT_DEBUG_PANEL), "GEDIT_DEBUG_PANEL", "debug-panel" },
        { C_FLAGS (GEDIT_DEBUG_PLUGINS), "GEDIT_DEBUG_PLUGINS", "debug-plugins" },
        { C_FLAGS (GEDIT_DEBUG_TAB), "GEDIT_DEBUG_TAB", "debug-tab" },
        { C_FLAGS (GEDIT_DEBUG_DOCUMENT), "GEDIT_DEBUG_DOCUMENT", "debug-document" },
        { C_FLAGS (GEDIT_DEBUG_COMMANDS), "GEDIT_DEBUG_COMMANDS", "debug-commands" },
        { C_FLAGS (GEDIT_DEBUG_APP), "GEDIT_DEBUG_APP", "debug-app" },
        { C_FLAGS (GEDIT_DEBUG_UTILS), "GEDIT_DEBUG_UTILS", "debug-utils" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GeditDebugSection"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gedit-tab.h" */

GType
gedit_tab_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GEDIT_TAB_STATE_NORMAL), "GEDIT_TAB_STATE_NORMAL", "normal" },
        { C_ENUM (GEDIT_TAB_STATE_LOADING), "GEDIT_TAB_STATE_LOADING", "loading" },
        { C_ENUM (GEDIT_TAB_STATE_REVERTING), "GEDIT_TAB_STATE_REVERTING", "reverting" },
        { C_ENUM (GEDIT_TAB_STATE_SAVING), "GEDIT_TAB_STATE_SAVING", "saving" },
        { C_ENUM (GEDIT_TAB_STATE_PRINTING), "GEDIT_TAB_STATE_PRINTING", "printing" },
        { C_ENUM (GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW), "GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW", "showing-print-preview" },
        { C_ENUM (GEDIT_TAB_STATE_LOADING_ERROR), "GEDIT_TAB_STATE_LOADING_ERROR", "loading-error" },
        { C_ENUM (GEDIT_TAB_STATE_REVERTING_ERROR), "GEDIT_TAB_STATE_REVERTING_ERROR", "reverting-error" },
        { C_ENUM (GEDIT_TAB_STATE_SAVING_ERROR), "GEDIT_TAB_STATE_SAVING_ERROR", "saving-error" },
        { C_ENUM (GEDIT_TAB_STATE_GENERIC_ERROR), "GEDIT_TAB_STATE_GENERIC_ERROR", "generic-error" },
        { C_ENUM (GEDIT_TAB_STATE_CLOSING), "GEDIT_TAB_STATE_CLOSING", "closing" },
        { C_ENUM (GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION), "GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION", "externally-modified-notification" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GeditTabState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gedit-window.h" */

GType
gedit_window_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GEDIT_WINDOW_STATE_NORMAL), "GEDIT_WINDOW_STATE_NORMAL", "normal" },
        { C_FLAGS (GEDIT_WINDOW_STATE_SAVING), "GEDIT_WINDOW_STATE_SAVING", "saving" },
        { C_FLAGS (GEDIT_WINDOW_STATE_PRINTING), "GEDIT_WINDOW_STATE_PRINTING", "printing" },
        { C_FLAGS (GEDIT_WINDOW_STATE_LOADING), "GEDIT_WINDOW_STATE_LOADING", "loading" },
        { C_FLAGS (GEDIT_WINDOW_STATE_ERROR), "GEDIT_WINDOW_STATE_ERROR", "error" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GeditWindowState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

