
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "gedit-notebook.h" */

GType gedit_notebook_show_tabs_mode_type_get_type (void);
#define GEDIT_TYPE_NOTEBOOK_SHOW_TABS_MODE_TYPE (gedit_notebook_show_tabs_mode_type_get_type())

G_END_DECLS

/* Generated data ends here */

